@@ -76,13 +76,20 @@ public:
     return r;
   }
 
-  int exit(int r) {
+  int signal_exit(int r) {
     if (forked) {
-      // tell parent
-      (void)::write(fd[1], &r, sizeof(r));
+      // tell parent.  this shouldn't fail, but if it does, pass the
+      // error back to the parent.
+      int ret = safe_write(fd[1], &r, sizeof(r));
+      if (ret <= 0)
+	return ret;
     }
     return r;
   }
+  void exit(int r) {
+    signal_exit(r);
+    ::exit(r);
+  }
 
   void daemonize() {
     assert(forked);
