@@ -7,7 +7,7 @@
 #define dout_subsys ceph_subsys_crush
 
 
-void CrushWrapper::find_roots(set<int>& roots) const
+void CrushWrapper::find_takes(set<int>& roots) const
 {
   for (unsigned i=0; i<crush->max_rules; i++) {
     crush_rule *r = crush->rules[i];
@@ -20,6 +20,17 @@ void CrushWrapper::find_roots(set<int>& roots) const
   }
 }
 
+void CrushWrapper::find_roots(set<int>& roots) const
+{
+  for (int i = 0; i < crush->max_buckets; i++) {
+    if (!crush->buckets[i])
+      continue;
+    crush_bucket *b = crush->buckets[i];
+    if (!_search_item_exists(b->id))
+      roots.insert(b->id);
+  }
+}
+
 bool CrushWrapper::subtree_contains(int root, int item) const
 {
   if (root == item)
@@ -39,14 +50,41 @@ bool CrushWrapper::subtree_contains(int root, int item) const
   return false;
 }
 
-
-int CrushWrapper::remove_item(CephContext *cct, int item)
+bool CrushWrapper::_maybe_remove_last_instance(CephContext *cct, int item, bool unlink_only)
 {
-  ldout(cct, 5) << "remove_item " << item << dendl;
+  // last instance?
+  if (_search_item_exists(item)) {
+    return false;
+  }
+
+  if (item < 0 && !unlink_only) {
+    crush_bucket *t = get_bucket(item);
+    ldout(cct, 5) << "_maybe_remove_last_instance removing bucket " << item << dendl;
+    crush_remove_bucket(crush, t);
+  }
+  if ((item >= 0 || !unlink_only) && name_map.count(item)) {
+    ldout(cct, 5) << "_maybe_remove_last_instance removing name for item " << item << dendl;
+    name_map.erase(item);
+    have_rmaps = false;
+  }
+  return true;
+}
+
+int CrushWrapper::remove_item(CephContext *cct, int item, bool unlink_only)
+{
+  ldout(cct, 5) << "remove_item " << item << (unlink_only ? " unlink_only":"") << dendl;
 
-  crush_bucket *was_bucket = 0;
   int ret = -ENOENT;
 
+  if (item < 0 && !unlink_only) {
+    crush_bucket *t = get_bucket(item);
+    if (t && t->size) {
+      ldout(cct, 1) << "remove_item bucket " << item << " has " << t->size
+		    << " items, not empty" << dendl;
+      return -ENOTEMPTY;
+    }
+  }
+
   for (int i = 0; i < crush->max_buckets; i++) {
     if (!crush->buckets[i])
       continue;
@@ -55,41 +93,95 @@ int CrushWrapper::remove_item(CephContext *cct, int item)
     for (unsigned i=0; i<b->size; ++i) {
       int id = b->items[i];
       if (id == item) {
-	if (item < 0) {
-	  crush_bucket *t = get_bucket(item);
-	  if (t && t->size) {
-	    ldout(cct, 1) << "remove_device bucket " << item << " has " << t->size << " items, not empty" << dendl;
-	    return -ENOTEMPTY;
-	  }	    
-	  was_bucket = t;
-	}
 	adjust_item_weight(cct, item, 0);
-	ldout(cct, 5) << "remove_device removing item " << item << " from bucket " << b->id << dendl;
+	ldout(cct, 5) << "remove_item removing item " << item
+		      << " from bucket " << b->id << dendl;
 	crush_bucket_remove_item(b, item);
 	ret = 0;
       }
     }
   }
 
-  if (was_bucket) {
-    ldout(cct, 5) << "remove_device removing bucket " << item << dendl;
-    crush_remove_bucket(crush, was_bucket);
-  }
-  if (item >= 0 && name_map.count(item)) {
-    name_map.erase(item);
-    have_rmaps = false;
+  if (_maybe_remove_last_instance(cct, item, unlink_only))
     ret = 0;
-  }
   
   return ret;
 }
 
+bool CrushWrapper::_search_item_exists(int item) const
+{
+  for (int i = 0; i < crush->max_buckets; i++) {
+    if (!crush->buckets[i])
+      continue;
+    crush_bucket *b = crush->buckets[i];
+    for (unsigned i=0; i<b->size; ++i) {
+      if (b->items[i] == item)
+	return true;
+    }
+  }
+  return false;
+}
+
+int CrushWrapper::_remove_item_under(CephContext *cct, int item, int ancestor, bool unlink_only)
+{
+  ldout(cct, 5) << "_remove_item_under " << item << " under " << ancestor
+		<< (unlink_only ? " unlink_only":"") << dendl;
+
+  if (ancestor >= 0) {
+    return -EINVAL;
+  }
+
+  if (!bucket_exists(ancestor))
+    return -EINVAL;
+
+  int ret = -ENOENT;
+
+  crush_bucket *b = get_bucket(ancestor);
+  for (unsigned i=0; i<b->size; ++i) {
+    int id = b->items[i];
+    if (id == item) {
+      adjust_item_weight(cct, item, 0);
+      ldout(cct, 5) << "_remove_item_under removing item " << item << " from bucket " << b->id << dendl;
+      crush_bucket_remove_item(b, item);
+      ret = 0;
+    } else if (id < 0) {
+      int r = remove_item_under(cct, item, id, unlink_only);
+      if (r == 0)
+	ret = 0;
+    }
+  }
+  return ret;
+}
+
+int CrushWrapper::remove_item_under(CephContext *cct, int item, int ancestor, bool unlink_only)
+{
+  ldout(cct, 5) << "remove_item_under " << item << " under " << ancestor
+		<< (unlink_only ? " unlink_only":"") << dendl;
+  int ret = _remove_item_under(cct, item, ancestor, unlink_only);
+  if (ret < 0)
+    return ret;
+
+  if (item < 0 && !unlink_only) {
+    crush_bucket *t = get_bucket(item);
+    if (t && t->size) {
+      ldout(cct, 1) << "remove_item_undef bucket " << item << " has " << t->size
+		    << " items, not empty" << dendl;
+      return -ENOTEMPTY;
+    }
+  }
+
+  if (_maybe_remove_last_instance(cct, item, unlink_only))
+    ret = 0;
+
+  return ret;
+}
+
 bool CrushWrapper::check_item_loc(CephContext *cct, int item, const map<string,string>& loc,
 				  int *weight)
 {
   ldout(cct, 5) << "check_item_loc item " << item << " loc " << loc << dendl;
 
-  for (map<int,string>::const_iterator p = type_map.begin(); p != type_map.end(); p++) {
+  for (map<int,string>::const_iterator p = type_map.begin(); p != type_map.end(); ++p) {
     // ignore device
     if (p->first == 0)
       continue;
@@ -133,35 +225,48 @@ bool CrushWrapper::check_item_loc(CephContext *cct, int item, const map<string,s
   return false;
 }
 
-map<string, string> CrushWrapper::get_full_location(int id){
-
+map<string, string> CrushWrapper::get_full_location(int id)
+{
+  vector<pair<string, string> > full_location_ordered;
   map<string,string> full_location;
+
+  get_full_location_ordered(id, full_location_ordered);
+
+  std::copy(full_location_ordered.begin(),
+      full_location_ordered.end(),
+      std::inserter(full_location, full_location.begin()));
+
+  return full_location;
+}
+
+int CrushWrapper::get_full_location_ordered(int id, vector<pair<string, string> >& path)
+{
+  int parent_id, ret;
   pair<string, string> parent_coord;
-  parent_coord = get_immediate_parent(id);
-  int parent_id;
+  parent_coord = get_immediate_parent(id, &ret);
 
   // read the type map and get the name of the type with the largest ID
   int high_type = 0;
-  for (map<int, string>::iterator it = type_map.begin(); it != type_map.end(); it++){
+  for (map<int, string>::iterator it = type_map.begin(); it != type_map.end(); ++it){
     if ( (*it).first > high_type )
       high_type = (*it).first;
   }
 
   string high_type_name = type_map[high_type];
 
-  full_location[ parent_coord.first ] = parent_coord.second;
+  path.push_back(parent_coord);
   parent_id = get_item_id( (parent_coord.second).c_str() );
 
 
   while (parent_coord.first != high_type_name) {
     parent_coord = get_immediate_parent(parent_id);
-    full_location[ parent_coord.first ] = parent_coord.second;
+    path.push_back(parent_coord);
     if ( parent_coord.first != high_type_name ){
       parent_id = get_item_id( (parent_coord.second).c_str() );
     }
   }
 
-  return full_location;
+  return ret;
 }
 
 
@@ -181,7 +286,7 @@ map<int, string> CrushWrapper::get_parent_hierarchy(int id)
 
   // read the type map and get the name of the type with the largest ID
   int high_type = 0;
-  for (map<int, string>::iterator it = type_map.begin(); it != type_map.end(); it++){
+  for (map<int, string>::iterator it = type_map.begin(); it != type_map.end(); ++it){
     if ( (*it).first > high_type )
       high_type = (*it).first;
   }
@@ -228,17 +333,19 @@ int CrushWrapper::insert_item(CephContext *cct, int item, float weight, string n
   ldout(cct, 5) << "insert_item item " << item << " weight " << weight
 		<< " name " << name << " loc " << loc << dendl;
 
-  if (name_exists(name.c_str())) {
-    ldout(cct, 1) << "error: device name '" << name << "' already exists as id "
-		  << get_item_id(name.c_str()) << dendl;
-    return -EEXIST;
+  if (name_exists(name)) {
+    if (get_item_id(name) != item) {
+      ldout(cct, 10) << "device name '" << name << "' already exists as id "
+		     << get_item_id(name) << dendl;
+      return -EEXIST;
+    }
+  } else {
+    set_item_name(item, name);
   }
 
-  set_item_name(item, name.c_str());
-
   int cur = item;
 
-  for (map<int,string>::iterator p = type_map.begin(); p != type_map.end(); p++) {
+  for (map<int,string>::iterator p = type_map.begin(); p != type_map.end(); ++p) {
     // ignore device type
     if (p->first == 0)
       continue;
@@ -290,6 +397,13 @@ int CrushWrapper::insert_item(CephContext *cct, int item, float weight, string n
 	return -EEXIST;
       }
     
+    // are we forming a loop?
+    if (subtree_contains(cur, b->id)) {
+      ldout(cct, 1) << "insert_item " << cur << " already contains " << b->id
+		    << "; cannot form loop" << dendl;
+      return -ELOOP;
+    }
+
     ldout(cct, 5) << "insert_item adding " << cur << " weight " << weight
 		  << " to bucket " << id << dendl;
     int r = crush_bucket_add_item(b, cur, 0);
@@ -334,6 +448,26 @@ int CrushWrapper::move_bucket(CephContext *cct, int id, const map<string,string>
   return insert_item(cct, id, bucket_weight / (float)0x10000, id_name, loc);
 }
 
+int CrushWrapper::link_bucket(CephContext *cct, int id, const map<string,string>& loc)
+{
+  // sorry this only works for buckets
+  if (id >= 0)
+    return -EINVAL;
+
+  if (!item_exists(id))
+    return -ENOENT;
+
+  // get the name of the bucket we are trying to move for later
+  string id_name = get_item_name(id);
+
+  // detach the bucket
+  crush_bucket *b = get_bucket(id);
+  unsigned bucket_weight = b->weight;
+
+  // insert the bucket back into the hierarchy
+  return insert_item(cct, id, bucket_weight / (float)0x10000, id_name, loc);
+}
+
 int CrushWrapper::create_or_move_item(CephContext *cct, int item, float weight, string name,
 				      const map<string,string>& loc)  // typename -> bucketname
 {
@@ -345,7 +479,7 @@ int CrushWrapper::create_or_move_item(CephContext *cct, int item, float weight,
     if (item_exists(item)) {
       weight = get_item_weightf(item);
       ldout(cct, 10) << "create_or_move_item " << item << " exists with weight " << weight << dendl;
-      remove_item(cct, item);
+      remove_item(cct, item, true);
     }
     ldout(cct, 5) << "create_or_move_item adding " << item << " weight " << weight
 		  << " at " << loc << dendl;
@@ -376,12 +510,12 @@ int CrushWrapper::update_item(CephContext *cct, int item, float weight, string n
     }
     if (get_item_name(item) != name) {
       ldout(cct, 5) << "update_item setting " << item << " name to " << name << dendl;
-      set_item_name(item, name.c_str());
+      set_item_name(item, name);
       ret = 1;
     }
   } else {
     if (item_exists(item)) {
-      remove_item(cct, item);
+      remove_item(cct, item, true);
     }
     ldout(cct, 5) << "update_item adding " << item << " weight " << weight
 		  << " at " << loc << dendl;
@@ -408,19 +542,23 @@ int CrushWrapper::get_item_weight(int id)
 int CrushWrapper::adjust_item_weight(CephContext *cct, int id, int weight)
 {
   ldout(cct, 5) << "adjust_item_weight " << id << " weight " << weight << dendl;
+  int changed = 0;
   for (int bidx = 0; bidx < crush->max_buckets; bidx++) {
     crush_bucket *b = crush->buckets[bidx];
     if (b == 0)
       continue;
-    for (unsigned i = 0; i < b->size; i++)
+    for (unsigned i = 0; i < b->size; i++) {
       if (b->items[i] == id) {
 	int diff = crush_bucket_adjust_item_weight(b, id, weight);
-	ldout(cct, 5) << "adjust_item_weight " << id << " diff " << diff << dendl;
+	ldout(cct, 5) << "adjust_item_weight " << id << " diff " << diff << " in bucket " << bidx << dendl;
 	adjust_item_weight(cct, -1 - bidx, b->weight);
-	return 0;
+	changed++;
       }
+    }
   }
-  return -ENOENT;
+  if (!changed)
+    return -ENOENT;
+  return changed;
 }
 
 bool CrushWrapper::check_item_present(int id)
@@ -439,9 +577,10 @@ bool CrushWrapper::check_item_present(int id)
 }
 
 
-pair<string,string> CrushWrapper::get_immediate_parent(int id)
+pair<string,string> CrushWrapper::get_immediate_parent(int id, int *_ret)
 {
   pair <string, string> loc;
+  int ret = -ENOENT;
 
   for (int bidx = 0; bidx < crush->max_buckets; bidx++) {
     crush_bucket *b = crush->buckets[bidx];
@@ -452,9 +591,13 @@ pair<string,string> CrushWrapper::get_immediate_parent(int id)
         string parent_id = name_map[b->id];
         string parent_bucket_type = type_map[b->type];
         loc = make_pair(parent_bucket_type, parent_id);
+        ret = 0;
       }
   }
 
+  if (_ret)
+    *_ret = ret;
+
   return loc;
 }
 
@@ -478,7 +621,7 @@ void CrushWrapper::reweight(CephContext *cct)
 {
   set<int> roots;
   find_roots(roots);
-  for (set<int>::iterator p = roots.begin(); p != roots.end(); p++) {
+  for (set<int>::iterator p = roots.begin(); p != roots.end(); ++p) {
     if (*p >= 0)
       continue;
     crush_bucket *b = get_bucket(*p);
@@ -492,12 +635,12 @@ int CrushWrapper::add_simple_rule(string name, string root_name, string failure_
 {
   if (rule_exists(name))
     return -EEXIST;
-  if (!name_exists(root_name.c_str()))
+  if (!name_exists(root_name))
     return -ENOENT;
-  int root = get_item_id(root_name.c_str());
+  int root = get_item_id(root_name);
   int type = 0;
   if (failure_domain_name.length()) {
-    type = get_type_id(failure_domain_name.c_str());
+    type = get_type_id(failure_domain_name);
     if (type <= 0) // bah, returns 0 on error; but its ok, device isn't a domain really
       return -EINVAL;
   }
@@ -525,7 +668,7 @@ int CrushWrapper::add_simple_rule(string name, string root_name, string failure_
 			0);
   crush_rule_set_step(rule, 2, CRUSH_RULE_EMIT, 0, 0);
   int rno = crush_add_rule(crush, rule, -1);
-  set_rule_name(rno, name.c_str());
+  set_rule_name(rno, name);
   have_rmaps = false;
   return rno;
 }
