@@ -33,7 +33,6 @@
 
 #include "common/config.h"
 #include "include/assert.h"
-#include <sstream>
 
 #define dout_subsys ceph_subsys_mon
 #undef dout_prefix
@@ -49,15 +48,15 @@ ostream& operator<<(ostream& out, AuthMonitor& pm)
   return out << "auth";
 }
 
-void AuthMonitor::check_rotate()
+bool AuthMonitor::check_rotate()
 {
   KeyServerData::Incremental rot_inc;
   rot_inc.op = KeyServerData::AUTH_INC_SET_ROTATING;
   if (!mon->key_server.updated_rotating(rot_inc.rotating_bl, last_rotating_ver))
-    return;
-  dout(10) << "AuthMonitor::tick() updated rotating, now calling propose_pending" << dendl;
+    return false;
+  dout(10) << __func__ << " updated rotating" << dendl;
   push_cephx_inc(rot_inc);
-  propose_pending();
+  return true;
 }
 
 /*
@@ -68,12 +67,12 @@ void AuthMonitor::tick()
 {
   if (!is_active()) return;
 
-  update_from_paxos();
   dout(10) << *this << dendl;
 
   if (!mon->is_leader()) return; 
 
-  check_rotate();
+  if (check_rotate())
+    propose_pending();
 }
 
 void AuthMonitor::on_active()
@@ -92,6 +91,11 @@ void AuthMonitor::create_initial()
 {
   dout(10) << "create_initial -- creating initial map" << dendl;
 
+  // initialize rotating keys
+  last_rotating_ver = 0;
+  check_rotate();
+  assert(pending_auth.size() == 1);
+
   KeyRing keyring;
   bufferlist bl;
   int ret = mon->store->get("mkfs", "keyring", bl);
@@ -109,7 +113,7 @@ void AuthMonitor::create_initial()
   pending_auth.push_back(inc);
 }
 
-void AuthMonitor::update_from_paxos()
+void AuthMonitor::update_from_paxos(bool *need_bootstrap)
 {
   dout(10) << __func__ << dendl;
   version_t version = get_version();
