@@ -114,7 +114,6 @@ void PGMonitor::tick()
 {
   if (!is_active()) return;
 
-  update_from_paxos();
   handle_osd_timeouts();
 
   if (mon->is_leader()) {
@@ -136,7 +135,7 @@ void PGMonitor::create_initial()
   dout(10) << "create_initial -- creating initial map" << dendl;
 }
 
-void PGMonitor::update_from_paxos()
+void PGMonitor::update_from_paxos(bool *need_bootstrap)
 {
   version_t version = get_version();
   if (version == pg_map.version)
@@ -220,7 +219,7 @@ void PGMonitor::update_from_paxos()
   update_logger();
 }
 
-void PGMonitor::init()
+void PGMonitor::post_paxos_update()
 {
   if (mon->osdmon()->osdmap.get_epoch()) {
     map_pg_creates();
