@@ -180,11 +180,12 @@ struct RGWObjState {
   bool has_data;
   bufferlist data;
   bool prefetch_data;
+  bool keep_tail;
 
   map<string, bufferlist> attrset;
   RGWObjState() : is_atomic(false), has_attrs(0), exists(false),
                   size(0), mtime(0), epoch(0), fake_tag(false), has_manifest(false),
-                  has_data(false), prefetch_data(false) {}
+                  has_data(false), prefetch_data(false), keep_tail(false) {}
 
   bool get_attr(string name, bufferlist& dest) {
     map<string, bufferlist>::iterator iter = attrset.find(name);
@@ -238,7 +239,7 @@ struct RGWPoolIterCtx {
   librados::ObjectIterator iter;
 };
 
-struct RGWRadosParams {
+struct RGWZoneParams {
   rgw_bucket domain_root;
   rgw_bucket control_pool;
   rgw_bucket gc_pool;
@@ -253,6 +254,7 @@ struct RGWRadosParams {
 
   int init(CephContext *cct, RGWRados *store);
   void init_default();
+  int store_info(CephContext *cct, RGWRados *store);
 
   void encode(bufferlist& bl) const {
     ENCODE_START(1, 1, bl);
@@ -284,8 +286,9 @@ struct RGWRadosParams {
     DECODE_FINISH(bl);
   }
   void dump(Formatter *f) const;
+  void decode_json(JSONObj *obj);
 };
-WRITE_CLASS_ENCODER(RGWRadosParams);
+WRITE_CLASS_ENCODER(RGWZoneParams);
   
 class RGWRados
 {
@@ -390,7 +393,7 @@ public:
                cct(NULL), rados(NULL),
                pools_initialized(false) {}
 
-  RGWRadosParams params;
+  RGWZoneParams zone;
 
   virtual ~RGWRados() {
     if (rados) {
@@ -559,8 +562,9 @@ public:
    * Returns 0 on success, -ERR# otherwise.
    */  virtual int delete_bucket(rgw_bucket& bucket);
 
-  virtual int set_buckets_enabled(std::vector<rgw_bucket>& buckets, bool enabled);
-  virtual int bucket_suspended(rgw_bucket& bucket, bool *suspended);
+  int set_bucket_owner(rgw_bucket& bucket, ACLOwner& owner);
+  int set_buckets_enabled(std::vector<rgw_bucket>& buckets, bool enabled);
+  int bucket_suspended(rgw_bucket& bucket, bool *suspended);
 
   /** Delete an object.*/
   virtual int delete_obj(void *ctx, rgw_obj& src_obj);
@@ -658,6 +662,7 @@ public:
                        uint64_t *epoch, map<string, bufferlist> *attrs, bufferlist *first_chunk);
 
   virtual bool supports_omap() { return true; }
+  int omap_get_vals(rgw_obj& obj, bufferlist& header, const std::string& marker, uint64_t count, std::map<string, bufferlist>& m);
   virtual int omap_get_all(rgw_obj& obj, bufferlist& header, std::map<string, bufferlist>& m);
   virtual int omap_set(rgw_obj& obj, std::string& key, bufferlist& bl);
   virtual int omap_set(rgw_obj& obj, map<std::string, bufferlist>& m);
